using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

using TAPI;
using Terraria;

using FearTheReaper.Items;
using FearTheReaper.NPCs;

//using RoboUtils;


namespace FearTheReaper {
	public class FTR_Player : TAPI.ModPlayer {
		/**	Timer used for counting off whole seconds. */
		private Stopwatch WholeSecondClock = new Stopwatch();

		/**	Total time (milliseconds) of remaining day remaining in previous game. */
		public static long ElapsedDayTimeBefore = 0L;


		/**	A timer to run for counting day-lengths gone by. */
		private Stopwatch DayLengthClock = new Stopwatch();

		/**	A timer to run to count up to Ferryman bribe duration. */
		private Stopwatch FerrymanBribeClock = new Stopwatch();



		private bool isSinner = true;	// We're born sinners!
		/**	Indicates condition for Reaper and Ferryman to both be able to spawn. */
		public bool IsSinner { get { return isSinner; } }

		private bool isAbjected = false;
		/**	Indicates reaper is able to randomly spawn for current player. */
		public bool IsAbjected { get { return isAbjected; } }

		private bool hasRepented = false;
		/**	Indicates player has (at least once) repented (removed) their sins. */
		public bool HasRepented { get { return hasRepented; } }


		/**	Indicates how dangerous the reaper will be to us (ascending; from 0 to 10). */
		private int reaperRage = 0;
		public int ReaperRage {
			get { return reaperRage; }
			//RoboUtils.MsgSelf("Reaper Rage: "+value);
			set { reaperRage = (int)MathHelper.Clamp( value, 0, 10 ); }
		}


		/**	Usual time of Ferryman's bribe effect (in milliseconds). */
		public readonly static long FerrymanBribeBaseTime = (long)(RoboUtils.FullDayDuration * 0.65f) * 1000L;

		/**	Time to add to Ferryman's bribe elapsed measure (in milliseconds). */
		private long FerrymanBribeAddedElapsed = (long)((float)FerrymanBribeBaseTime * 0.875f);	// 1/8 normal bribe time


		/**	Indicates if player has chatted to Ferryman since their sin. */
		public bool HasFerrymanChatSinceLastSin = false;


		/**	Initial Ferryman bribe price. */
		public static int BaseFerryPrice = 5000;	// 5k copper coins, AKA 50 silver coins

		/**	Ferryman's bribe price scale amount. */
		public static float FerryPriceScale = 1.2f;

		private int ferryPrice = BaseFerryPrice;
		/**	Current bribe price the ferryman will charge. */
		public int FerryPrice { get { return ferryPrice; } }


		private int _lingerMeasure = 0;
		private int lingerMeasure {
			get { return _lingerMeasure; }
			set { _lingerMeasure = (int)Math.Max( value, 0 ); }
		}
		/**	Measures time (in seconds) a player spends lingering in a general area. */
		public int LingerMeasure { get { return _lingerMeasure; } }

		/**	Point where player is tested to be 'lingering'. */
		private Vector2 LingerPosition;



		/**	Indicates if player is 'anchored'; cannot (magically) teleport. */
		public bool IsAnchored = false;
		/**	Indicates if player is 'sanctified'; reaper cannot spawn. */
		public bool IsSanctified = false;




		/**	How much 'progress' can be expected for a player to be considered 'advanced' in the game. */
		public readonly static int AdvancedProgressMilestone = 99;

		private int _baseProgressMeasure;
		private int baseProgressMeasure {
			get { return _baseProgressMeasure; }
			set { _baseProgressMeasure = Math.Max( value, 0 ); }
		}
		/**	A measure of player's relative progress throughout the game. Many factors affect this. */
		public int BaseProgressMeasure { get { return _baseProgressMeasure; } }


		/**	Indicates player's equipped ferry token. */
		public FerryTokenAccessoryItem EquippedFerryToken = null;	// Lazy! Bad form!




		private bool _IsInited = false;
		private bool _IsLoaded = false;
		private bool _IsGameInited = false;

//        public override void Initialize() {
//            if( _IsInited ) { return; }	// Dunno.
//int i;
//for( i=0; i < 58; i++ ) {
//    if( player.inventory[i].type == 0 ) {
//        player.inventory[i].SetDefaults( "Enchanted Sword" );
//        player.inventory[i].Prefix( -1 );
//        break;
//    }
//}
//            _IsInited = true;
//        }


		public void LoadFromBinBuffer( BinBuffer bb ) {
			/*1*/	isSinner = bb.ReadBool();
			/*2*/	isAbjected = bb.ReadBool();
			/*3*/	reaperRage = bb.ReadInt();
			/*4*/	HasFerrymanChatSinceLastSin = bb.ReadBool();
			/*5*/	hasRepented = bb.ReadBool();
			/*6*/	ferryPrice = bb.ReadInt();
			/*7*/	FerrymanBribeAddedElapsed = bb.ReadLong();
			/*8*/	baseProgressMeasure = bb.ReadInt();
			/*9*/	ElapsedDayTimeBefore = bb.ReadLong();
RoboUtils.MsgSelf( "LOADING PLAYER", RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "Is Sinner: " + IsSinner + ",      Is Abjected: " + IsAbjected, RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "Reaper Rage: " + ReaperRage, RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "Has Repented: " + HasRepented + ",      Ferry Price: " + FerryPrice, RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "FerrymanBribeAddedElapsed: " + FerrymanBribeAddedElapsed + ",      Progress: " + BaseProgressMeasure, RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "Computed progress:"+ComputeProgressMeasure()+", ElapsedDayTimeBefore: " + ElapsedDayTimeBefore, RoboUtils.MsgType.Debug );
RoboUtils.MsgSelf( "LOADING PLAYER COMPLETE "+Main.netMode, RoboUtils.MsgType.Debug );
		}

		public void SaveToBinBuffer( BinBuffer bb ) {
			/*1*/	bb.Write( (bool)IsSinner );
			/*2*/	bb.Write( (bool)IsAbjected );
			/*3*/	bb.Write( (int)ReaperRage );
			/*4*/	bb.Write( (bool)HasFerrymanChatSinceLastSin );
			/*5*/	bb.Write( (bool)HasRepented );
			/*6*/	bb.Write( (int)FerryPrice );
			/*7*/	bb.Write( (long)ComputeElapsedBribeTime() );
			/*8*/	bb.Write( (int)BaseProgressMeasure );
			/*9*/	bb.Write( (long)ComputeElapsedDayTime() );
		}


		public override void Load( BinBuffer bb ) {
			//if( _IsLoaded ) { return; }

			LoadFromBinBuffer( bb );

			_IsLoaded = true;
		}

		public override void Save( BinBuffer bb ) {
			SaveToBinBuffer( bb );

			if( RoboUtils.IsDebug ) {
				DebugLog.Dump( "(player " + player.name + " netMode " + Main.netMode + ")" );
			}
		}


		public override void DamageNPC( Projectile projectile, NPC npc, int hitDir, ref int damage, ref float knockback, ref bool crit, ref float critMult ) {
			if( npc.displayName == "The Reaper" ) {
				// Enchanted Beam
				if( projectile.type == 173 ) {
					// Heals damage
					npc.life = Math.Min( npc.life + 3, npc.lifeMax+1 );
					// Reduces rage
					reaperRage -= 2;
					// Low knockback, though
					knockback = 1;
				}
			}
		}

		public override void DamageNPC( NPC npc, int hitDir, ref int damage, ref float knockback, ref bool crit, ref float critMult ) {
			if( npc.displayName == "The Reaper" ) {
				int held_item = player.inventory[player.selectedItem].type;
				// Enchanted Sword direct (rare underground item)
				if( held_item == 989 ) {
					//damage = 96;
					knockback = 6;
					//crit = false;
					npc.life = Math.Max( 0, npc.life-333 );
				}
			}
		}

		private void PostDamageMyNPC( NPC npc ) {
			if( RoboUtils.IsNPCDead( npc ) ) {
				if( npc.townNPC ) { KilledTownNPC(); }
				else if( npc.displayName == "The Reaper" ) { baseProgressMeasure++; }
			}
		}

		public override void DealtNPC( NPC npc, int hitDir, int dmgDealt, float knockback, bool crit ) {
			PostDamageMyNPC( npc );
		}
		public override void DealtNPC( Projectile projectile, NPC npc, int hitDir, int dmgDealt, float knockback, bool crit ) {
			PostDamageMyNPC( npc );
		}
		public override void PostKill( double damage, int hitDirection, bool pvp, string deathText ) {
			if( pvp ) {
				KilledSelf();
			} else {
				// Chill out, reaper
				if( RoboUtils.IsKillerByName( deathText, "The Reaper" ) ) {
					baseProgressMeasure -= 1;
					ReaperRage -= 5;
				}
			}
		}


		public override void PreUpdate() {
			if( Main.netMode != 2 ) { UpdateMe(false); }	// netMode == 2 never calls PreUpdate; ModWorld.PostUpdate substitutes!
		}

		public void UpdateMe( bool is_server_only ) {
			//if( (!_IsInited || !_IsLoaded) && !is_server_only ) { return; }	// Dunno.

			// Any other suitable place for this?
			if( !_IsGameInited ) {
				if( Main.netMode == 1 ) {
					BinBuffer data = new BinBuffer();

					data.Write( (int)player.whoAmI );
					SaveToBinBuffer( data );
					data.Pos = 0;
					
RoboUtils.MsgSelf( "Sending data... ("+data.BytesLeft+")", RoboUtils.MsgType.Debug );
					NetMessage.SendModData( Mods.GetMod("FearTheReaper"), 1, -1, -1, data );
				}
				_IsGameInited = true;
			}

			if( !WholeSecondClock.IsRunning ) {
				WholeSecondClock.Reset();
				WholeSecondClock.Start();
			}
			bool is_whole_second = WholeSecondClock.ElapsedMilliseconds >= 1000L;
			if( is_whole_second ) {
				WholeSecondClock.Reset();
				WholeSecondClock.Start();
			}

			// Death Sickles are not a smart thing to hold when the reaper is after you
			if( IsAbjected && player.inventory[player.selectedItem].type == 1327 ) {
				player.AddBuff( "Frozen", 2 );
			}

			UpdateHaunting();
			UpdateDayCounter();
			UpdateLinger( is_whole_second );
			UpdateSanctity( is_whole_second );
			if( is_whole_second ) { UpdateFailsafes(); }
		}



		private void UpdateHaunting() {
			// Ferryman bribe in effect: Run timer
			if( IsSinner && !IsAbjected && !FerrymanBribeClock.IsRunning ) {
				FerrymanBribeClock.Start();
			}

			// Ferryman bribe expired: (Re)start haunting
			if( FerrymanBribeClock.IsRunning && (ComputeElapsedBribeTime() >= FerrymanBribeBaseTime) ) {
				FerrymanBribeAddedElapsed = 0;
				FerrymanBribeClock.Stop();
				FerrymanBribeClock.Reset();
				BeginAbjection();
			}
		}

		private void UpdateDayCounter() {
			if( !DayLengthClock.IsRunning ) {
				DayLengthClock.Reset();
				DayLengthClock.Start();
			}

			long day_milliseconds = RoboUtils.FullDayDuration * 1000L;

			if( ComputeElapsedDayTime() >= day_milliseconds ) {
				DayLengthClock.Reset();
				DayLengthClock.Start();

				ElapsedDayTimeBefore = 0;

				baseProgressMeasure++;	// Each day gone by increases 'progress'
RoboUtils.MsgSelf("Day counted. Add progress.", RoboUtils.MsgType.Debug);
			}
		}


		private void UpdateLinger( bool is_whole_second ) {
			// Non-sinners need not be bothered
			if( !IsSinner ) { return; }

			if( is_whole_second ) {
				// Every 45s of lingering, update position
				if( LingerPosition == null || (LingerMeasure > 0 && (LingerMeasure % 45) == 0) ) {
RoboUtils.MsgSelf(" linger position updated ("+LingerMeasure+"%45="+(LingerMeasure % 45)+") "+(LingerPosition==null), RoboUtils.MsgType.Debug);
					LingerPosition = new Vector2( player.position.X, player.position.Y );
				}

				// Every 1s, increment linger count
				lingerMeasure++;
			}

			float distance = Vector2.Distance( player.position, LingerPosition ) / 16f;
			// If player moves to new area, they are lingering no longer
			if( distance > ReaperMobNPC.AuraVeryFar ) {
				LingerPosition = new Vector2( player.position.X, player.position.Y );

				// Reduce (not reset) linger count
				int linger_drop = (int)Math.Max( (float)LingerMeasure / 2f, 12f );
				lingerMeasure -= linger_drop;
RoboUtils.MsgSelf("Unlingered. ("+LingerMeasure+" = Old LingerMeasure - "+linger_drop+")", RoboUtils.MsgType.Debug);
			}
		}


		private bool _NearReaperStatue = false;

		private void UpdateSanctity( bool is_whole_second ) {
			// Non-sinners need not be bothered
			if( !IsSinner ) { return; }

			// If we're under bribe effect, indicate sanctity
			if( !IsAbjected ) {
				player.AddBuff( "FearTheReaper:SanctifiedBuff", 2 );
				return;
			}

			if( is_whole_second ) {
				int x_where = 0;
				int y_where = 0;

				// Any statues near?
				while( true ) {
					Tile tile = RoboUtils.GetNextScreenTileOfType( 105, ref x_where, ref y_where );
					if( tile == null ) {
						_NearReaperStatue = false;
						break;
					}

					// Any reaper statues near?
					if( RoboUtils.GetTileStyle2xX( tile ) == 11 ) {
						_NearReaperStatue = true;
						break;
					}
				}
			}

			if( _NearReaperStatue ) {
				player.AddBuff( "FearTheReaper:SanctifiedBuff", 2 );
				return;
			}
		}

		private void UpdateFailsafes() {
			Dictionary<int, int> buff_map = RoboUtils.GetBuffs( player );
			int anchored_type = BuffDef.byName["FearTheReaper:AnchoredBuff"];
			int sanctified_type = BuffDef.byName["FearTheReaper:SanctifiedBuff"];

			if( IsAnchored && !buff_map.ContainsKey( anchored_type ) ) {
				IsAnchored = false;
			}
			if( IsSanctified && !buff_map.ContainsKey( sanctified_type ) ) {
				IsSanctified = false;
			}
		}



		public long ComputeElapsedBribeTime() {
			return FerrymanBribeClock.ElapsedMilliseconds + FerrymanBribeAddedElapsed;
		}
		public long ComputeRemainingBribeTime() {
			return FerrymanBribeBaseTime - ComputeElapsedBribeTime();
		}

		public int ComputeProgressMeasure() {
			// Player's max health (i.e. each life crystal) adds progress
			int hp_progress = Math.Max(player.statLifeMax - 100, 0) / 30;
			int boss_progress = 0;
			int npc_progress = 0;

			if( NPC.downedBoss1 ) { boss_progress += 2; }
			if( NPC.downedBoss2 ) { boss_progress += 2; }
			if( NPC.downedBoss3 ) { boss_progress += 2; }
			if( NPC.downedQueenBee ) { boss_progress += 1; }
			if( NPC.downedPlantBoss ) { boss_progress += 3; }
			if( NPC.downedGolemBoss ) { boss_progress += 2; }
			if( NPC.downedFishron ) { boss_progress += 2; }
			if( NPC.downedMechBoss1 ) { boss_progress += 2; }
			if( NPC.downedMechBoss2 ) { boss_progress += 2; }
			if( NPC.downedMechBoss3 ) { boss_progress += 2; }
			if( Main.hardMode ) { boss_progress += 5; }

			if( NPC.downedGoblins ) { boss_progress += 1; }
			if( NPC.downedFrost ) { boss_progress += 1; }
			if( NPC.downedPirates ) { boss_progress += 1; }

			if( NPC.savedGoblin ) { npc_progress++; }
			if( NPC.savedWizard ) { npc_progress++; }
			if( NPC.savedMech ) { npc_progress++; }
			if( NPC.savedAngler ) { npc_progress++; }
			//if( NPC.savedStylist ) { npc_progress++; }	// "Progress"

			int max_progress = (int)((float)AdvancedProgressMilestone * 1.5f);
			int total_progress = BaseProgressMeasure + hp_progress + boss_progress + (npc_progress / 2);

			return Math.Min( total_progress, max_progress );
		}

		public long ComputeElapsedDayTime() {
			return ElapsedDayTimeBefore + DayLengthClock.ElapsedMilliseconds;
		}

		public int ComputeNewFerryPrice() {
			return (int)((float)FerryPrice * FerryPriceScale);
		}


		/**	Indicates if we allow a reaper to spawn. */
		public bool AllowsReaper() {
			if( !IsAbjected ) {
RoboUtils.MsgSelf( "Can't spawn: Player is not abjected. Bribe time remaining: " + ComputeRemainingBribeTime(), RoboUtils.MsgType.Debug );
				return false; }

			if( IsSanctified ) {
RoboUtils.MsgSelf( "Can't spawn: Player is sanctified.", RoboUtils.MsgType.Debug );
				return false; }

			if( RoboUtils.GetBuffs( player ).ContainsKey( BuffDef.byName["Vanilla:Invisibility"] ) ) {
				if( RoboUtils.IsPlayerNaked(player) ) {
RoboUtils.MsgSelf( "Can't spawn: Player is invisible.", RoboUtils.MsgType.Debug );
					return false; }
			}

			if( EquippedFerryToken != null ) {
RoboUtils.MsgSelf( "Can't spawn: Ferry token equipped.", RoboUtils.MsgType.Debug );
				return false; }

			return true;
		}



		private void KilledTownNPC() {
			ThouHathSinned();
		}
		private void KilledSelf() {
			//ThouHathSinned();
		}
		private void AfterBribe() {
			ThouHathSinned();
		}


		/**	Called when player commits any reaper-offending sin. Also a good way to anger the reaper. */
		private void ThouHathSinned() {
			/*
			isSinner = true;

			// This forces any current bribe to expire
			FerrymanBribeAddedElapsed = FerrymanBribeBaseTime;

			ReaperRage++;
			baseProgressMeasure++;
RoboUtils.MsgSelf("Thou hath sinned. Progress++", RoboUtils.MsgType.Debug);
			 */	// <- Currently not implemented
		}


		public bool BeginAbjection() {
			bool abjection_set = false;

			if( !IsAbjected && IsSinner ) {
				isAbjected = true;
				abjection_set = true;
			}

			if( abjection_set ) { RoboUtils.MsgSelf( "You feel abjected...", RoboUtils.MsgType.Warn ); }

			return abjection_set;
		}

		public bool CeaseAbjection() {
			bool abjection_unset = false;

			if( IsAbjected ) {
				if( ReaperMobNPC.ReapMyReaper( this, false ) ) {
RoboUtils.MsgSelf( player.name + "'s reaper reaped.", RoboUtils.MsgType.Debug );
				} else {
RoboUtils.MsgSelf( player.name + "'s reaper not reaped/reapable.", RoboUtils.MsgType.Debug );
				}

				isAbjected = false;
				abjection_unset = true;
			}

			if( abjection_unset ) { RoboUtils.MsgSelf( "Your abjectation subsides.", RoboUtils.MsgType.Alert ); }

			return abjection_unset;
		}


		/**	Tells the reaper to gtfo. Also calms him. */
		public void SatiateReaper() {
			// In all cases, reaper is calmed a bit
			ReaperRage -= 5;
			// Reaper haunting ceases
			CeaseAbjection();
RoboUtils.MsgSelf( "Reaper sated.", RoboUtils.MsgType.Debug );
		}


		/**	Tells the reaper to gtfo for good. */
		public void Repent() {
			ferryPrice = BaseFerryPrice;

			isSinner = false;
			isAbjected = false;
			HasFerrymanChatSinceLastSin = false;
			hasRepented = true;
			ReaperRage = 0;

			RoboUtils.MsgSelf( "You feel at peace.", RoboUtils.MsgType.Good );
		}


		/**	Pays Ferryman's toll. Purchase not implemented here. */
		private bool PayFerrymanMoney( int money ) {
			if( !player.BuyItem(money) ) { return false; }
			
			// Ferryman's fee increases, regardless of what was bought
			ferryPrice = ComputeNewFerryPrice();

			// TODO: Scale ferryman price based on reaper anger?

			return true;
		}

		/**	Pays Ferryman a token. Purchase not implemented here. */
		private bool PayFerrymanToken() {
			if( EquippedFerryToken == null ) { return false; }

			int slot = RoboUtils.GetAccessorySlotOfItem( player, EquippedFerryToken.item );
			if( slot == -1 ) { return false; }

			player.armor[slot].active = false;
			player.armor[slot] = new Item();
			EquippedFerryToken = null;

			return true;
		}


		/**	Attempts to pay toll produce its effect. */
		public bool BuyLifeToll() {
			if( !IsAbjected ) { return false; }

			if( !PayFerrymanMoney( FerryPrice ) ) { return false; }

			SatiateReaper();
			baseProgressMeasure++;
RoboUtils.MsgSelf( "Life toll; progress++", RoboUtils.MsgType.Debug );

			return true;
		}

		/**	Attempts to buy fate produce its effect. */
		public bool BuyFate() {
			if( !PayFerrymanToken() ) { return false; }

			Repent();

			return true;
		}
	}
}
