using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

using TAPI;
using Terraria;

//using RoboUtils;
using FearTheReaper;


namespace FearTheReaper.NPCs {
    public class FerrymanTownNPC : ModNPC {
		protected static readonly List<string> DialogNormal = new List<string>() {
			"The boundaries which divide Life from Death are at best shadowy and vague. Who shall say where the one ends, and where the other begins?",
			"All that we see or seem is but a dream within a dream.",
			"The good and the bad and the worst and the best have gone to their eternal rest.",
			"...and then there stole into my fancy, like a rich musical note, the thought of what sweet rest there must be in the grave.",
			"...the silence was unbroken, and the stillness gave no token.",
			"Believe only half of what you see and nothing that you hear."
			//"Sometimes I'm terrified of my heart; of its constant hunger for whatever it is it wants. The way it stops and starts."
		};
		protected static readonly List<string> DialogBloodmoon = new List<string>() {
			"Years of love have been forgot, in the hatred of a minute.",
			"There are moments when even to the sober eye of reason, the world of our sad humanity may assume the semblance of Hell.",
			"I do not suffer from insanity, I enjoy every minute of it.",
			"I felt that I breathed an atmosphere of sorrow."
		};
		


		private Stopwatch ExitEventClock = new Stopwatch();
		private Stopwatch EnterEventClock = new Stopwatch();
		private Stopwatch WetEventClock = new Stopwatch();


		private static bool? iAmCome = null;
		/**	Indicates if Ferryman exists on the map. */
		public static bool IAmCome {
			get {
				if( iAmCome != null ) { return (bool)iAmCome; }

				int ferryman_type = NPCDef.byName["FearTheReaper:FerrymanTownNPC"].type;

				// Find any Ferryman
				for( int i = 0; i < Main.npc.Length; i++ ) {
					NPC npc = Main.npc[i];
					if( npc == null || !npc.active ) { continue; }

					if( npc.type == ferryman_type ) {
						iAmCome = true;
						break;
					}
				}
				if( iAmCome == null ) { iAmCome = false; }

				return (bool)iAmCome;
			}
		}



		public override bool CanTownNPCSpawn() {
			bool can_ferry = false;

			// For each player
			for( int i = 0; i < Main.player.Length; i++ ) {
				Player player = Main.player[i];
				if( player == null || !player.active ) { continue; }

				FTR_Player ftr_player = player.GetSubClass<FTR_Player>();
				if( ftr_player == null ) { continue; }

				// If at least 1 is currently being haunted
				if( ftr_player.IsAbjected ) {
					can_ferry = true;	// The Ferryman Cometh!
					break;
				}
			}
			
			return can_ferry;
		}


		public override void OnSpawn() {
			EnterFerryman();
		}


		public override bool? CanHitFriendNPC( NPC npc ) { return false; } //doesn't get hurt by enemy npcs


		public override string SetName() {
			int rand = Main.rand.Next(5);
			string name = "";

			switch( rand ) {
			case 0:
				name = "Aken"; break;
			case 1:
				name = "Haros"; break;
			case 2:
				name = "Acheron"; break;
			//case 3:
			//    name = "Phlegethon"; break;
			//case 3:
			//    name = "Kokytos"; break;
			case 3:
				name = "Erebus"; break;
			case 4:
				//   name = "Styx"; break;
			//case 5:
			default:
				name = "Charon"; break;
			}

			return name;
        }


		public override string SetChat() {
			FTR_Player ftr_player = Main.localPlayer.GetSubClass<FTR_Player>();

			if( ftr_player.IsSinner && !ftr_player.HasFerrymanChatSinceLastSin ) {
				ftr_player.HasFerrymanChatSinceLastSin = true;
				if( !ftr_player.HasRepented ) {
					return "Thou who dreamt release from thy fate, bear it now evermore. Though maybe for a price, I...";
				} else {
					return "Your token did suffice, I say, but it shan't twice. Again, I ask my price.";
				}
			}

			List<string> dialog;
			if( !Main.bloodMoon ) { dialog = DialogNormal; }
			else { dialog = DialogBloodmoon; }

			int rand = Main.rand.Next( dialog.Count );
			return dialog[rand];
		}


		public const int HauntButtonId = 0;
		public const int SinnerButtonId = 1;

		public override void SetChatButtons( ref string[] buttons ) {
			FTR_Player ftr_player = Main.localPlayer.GetSubClass<FTR_Player>();

			if( ftr_player.IsSinner ) {
				if( ftr_player.IsAbjected ) {
					string rendered_price = RoboUtils.RenderMoney( ftr_player.FerryPrice );

					buttons[HauntButtonId] = "Pay Toll (" + rendered_price + ")";
					//buttons[1] = "Death Bribe ("+rendered_price+")";
				} else {
					buttons[HauntButtonId] = "...";
				}
			}

			if( ftr_player.IsSinner && ftr_player.EquippedFerryToken != null ) {
				buttons[SinnerButtonId] = "... (Give Token)";
			}
		}


		public override void OnChatButtonClicked( string[] buttons, int button_id, ref bool openShop ) {
			FTR_Player ftr_player = Main.localPlayer.GetSubClass<FTR_Player>();

			bool deal_made = false;

			switch( button_id ) {
			// Life toll
			case HauntButtonId:
				if( ftr_player.IsSinner && ftr_player.IsAbjected ) {
					deal_made = ftr_player.BuyLifeToll();

					if( deal_made ) {
						RoboUtils.UpdateCurrentNPCText( "Verily." );
					} else { RoboUtils.UpdateCurrentNPCText( "..." ); }
				} else { RoboUtils.UpdateCurrentNPCText( "..." ); }
				break;

			//// Death bribe
			//case DeathButtonId:
			//	deal_made = ftr_player.BuyDeathBribe();
			//
			//	if( deal_made ) {
			//		RoboUtils.UpdateCurrentNPCText( "Ehehe..." );
			//	} else { RoboUtils.UpdateCurrentNPCText( "..." ); }
			//	break;

			// Buy fate
			case SinnerButtonId:
				if( ftr_player.IsSinner && ftr_player.EquippedFerryToken != null ) {
					deal_made = ftr_player.BuyFate();

					if( deal_made ) {
						RoboUtils.UpdateCurrentNPCText( "A token trifle, but to buy one's fate, it will suffice. Farewell... for now!" );
					} else { RoboUtils.UpdateCurrentNPCText( "...?" ); }
				} else { RoboUtils.UpdateCurrentNPCText( "...?" ); }
				break;
			}
			
			// In all cases, if deal made, ferryman leaves (temporarily)
			if( deal_made ) { ExitFerryman(); }
		}



		private void EnterFerryman() {
			iAmCome = true;

			EnterEventClock.Reset();
			EnterEventClock.Start();
			WetEventClock.Start();
		}
		private void EnterFerrymanFinish() {
			//// For each player
			//for( int i = 0; i < Main.player.Length; i++ ) {
			//    if( Main.player[i] == null ) { continue; }
			//    Player player = Main.player[i];
			//    FTR_Player ftr_player = player.GetSubClass<FTR_Player>();
			//    if( ftr_player == null ) { continue; }

			//    // Resume haunting
			//    ftr_player.SetHaunt( true );
			//}
		}

		private void ExitFerryman() {
			iAmCome = false;

			ExitEventClock.Reset();
			ExitEventClock.Start();
		}
		private void ExitFerrymanFinish() {
			RoboUtils.DestroyNPC( npc );
		}



		public override void AI() {
			RunPostArriveEventTimer();
			RunPostDepartEventTimer();
			RunIdleEventTimer();
		}

		private void RunPostArriveEventTimer() {
			if( EnterEventClock.IsRunning && EnterEventClock.ElapsedMilliseconds >= 2000L ) {	// 2 seconds
				EnterFerrymanFinish();
				EnterEventClock.Stop();
				EnterEventClock.Reset();
			}
		}
		private void RunPostDepartEventTimer() {
			if( ExitEventClock.IsRunning && ExitEventClock.ElapsedMilliseconds >= 3500L ) {	// 3.5 seconds
				ExitFerrymanFinish();
				ExitEventClock.Stop();
				ExitEventClock.Reset();
			}
		}
		private void RunIdleEventTimer() {
			npc.dripping = true;
			//if( WetEventClock.IsRunning && WetEventClock.ElapsedMilliseconds >= 500L ) {	// 0.5 seconds
			//    npc.wet = true;
			//    WetEventClock.Stop();
			//    WetEventClock.Reset();
			//}
		}
	}
}
