using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

using TAPI;
using Terraria;


/**	This is just a collection of convenience and convention methods. Some are
 * likely redundant, but may minimize possible future refactoring/debugging.
 * Feel free to suggest improvements. */
public class RoboUtils {
	public static bool IsDebug = false;
	public static bool IsRNGDisabled = false;



	public enum MsgType { Good, Info, Alert, Warn, Debug };


	/**	Full day/night cycle duration (in seconds). */
	public static readonly long FullDayDuration = 1440;


	public static void MsgSelf( string text, MsgType msg_type = MsgType.Info ) {
		switch( msg_type ) {
		case MsgType.Good:
			Main.NewText( text, 100, 255, 100, false );
			break;
		case MsgType.Alert:
			Main.NewText( text, 230, 230, 45, false );
			break;
		case MsgType.Warn:
			Main.NewText( text, 175, 75, 255, false );
			break;
		case MsgType.Debug:
			if( !IsDebug ) { break; }
			Main.NewText( text, 144, 144, 144, false );
			break;
		case MsgType.Info:
		default:
			Main.NewText( text, 255, 255, 255, false );
			break;
		}

		if( IsDebug && msg_type == MsgType.Debug ) {
			DebugLog.Append( text );
		}
	}

	public static void MsgAll( string text, MsgType msg_type = MsgType.Info ) {
		if( Main.netMode == 0 ) {
			if( msg_type == MsgType.Debug ) {
				MsgSelf( "All: " + text, msg_type );
			} else {
				MsgSelf( text, msg_type );
			}
			return;
		}

		switch( msg_type ) {
		case MsgType.Good:
			NetMessage.SendData( 25, -1, -1, text, 255, 100f, 255f, 100f, 0f );
			break;
		case MsgType.Alert:
			NetMessage.SendData( 25, -1, -1, text, 255, 230f, 230f, 45f, 0f );
			break;
		case MsgType.Warn:
			NetMessage.SendData( 25, -1, -1, text, 255, 175f, 75f, 255f, 0f );
			break;
		case MsgType.Debug:
			if( !IsDebug ) { return; }
			text = "netMode " + Main.netMode + " for \"" + Main.localPlayer.name + "\": " + text;
			NetMessage.SendData( 25, -1, -1, text, 255, 144f, 144f, 144f, 0f );
			break;
		case MsgType.Info:
		default:
			NetMessage.SendData( 25, -1, -1, text, 255, 255f, 255f, 255f, 0f );
			break;
		}

		if( IsDebug && msg_type == MsgType.Debug ) {
			DebugLog.Append( "All: " + text );
		}
	}



	public static bool IsServer() {
		return Main.netMode == 0 || Main.netMode == 2;	// TODO: Verify correctness (i.e. what if player IS server?)
	}
	public static bool IsClient() {
		return Main.netMode == 0 || Main.netMode == 1;
	}


	public static bool IsNPCDead( NPC npc ) {
		return npc.life <= 0 || !npc.active;
	}


	public static bool IsBeingInvaded() {
		return Main.invasionType > 0 && Main.invasionDelay == 0 && Main.invasionSize > 0;
	}


	public static bool IsCurrentPlayerNearBoss() {
		Rectangle screen_zone = new Rectangle( (int)Main.screenPosition.X, (int)Main.screenPosition.Y, Main.screenWidth, Main.screenHeight );
		int boss_radius = 5000;

		for( int i = 0; i < Main.npc.Length; i++ ) {
			NPC npc = Main.npc[i];
			if( !npc.active || !npc.boss ) { continue; }

			int npc_left = (int)(npc.position.X + (float)npc.width / 2f) - boss_radius;
			int npc_top = (int)(npc.position.Y + (float)npc.height / 2f) - boss_radius;
			Rectangle npc_zone = new Rectangle( npc_left, npc_top, boss_radius*2, boss_radius*2 );

			if( screen_zone.Intersects(npc_zone) ) { return true; }
		}

		return false;
	}


	public static void DestroyNPC( NPC npc ) {
		npc.active = false;

		for( int i = 0; i < Main.npc.Length; i++ ) {
			if( Main.npc[i] == null ) { continue; }
			if( Main.npc[i].whoAmI == npc.whoAmI ) {
				NetMessage.SendData( 23, -1, -1, "", i, 0f, 0f, 0f, 0f );
				break;
			}
		}
	}


	public static void DestroyItem( Item item ) {
		item.active = false;
		item.type = 0;
		item.name = "";
		item.stack = 0;
	}

	public static void DestroyWorldItem( int i ) {
		Item item = Main.item[i];
		DestroyItem( item );

		if( Main.netMode != 0 ) {
			NetMessage.SendData( 21, -1, -1, "", i, 0f, 0f, 0f, 0f );
		}
	}


	public static int GetAccessorySlotOfItem( Player player, Item cmp_item ) {
		for( int i = 3; i <= 7; i++ ) {
			if( player.armor[i].IsNotTheSameAs(cmp_item) ) { continue; }
			return i;
		}
		return -1;
	}


	public static List<int> GetNpcWhosByType( int type ) {
		List<int> npcs = new List<int>();
		for( int i = 0; i < Main.npc.Length; i++ ) {
			if( Main.npc[i].active && Main.npc[i].type == type ) {
				npcs.Add( i );
			}
		}
		return npcs;
	}


	public static void UpdateCurrentNPCText( string text ) {
		Main.npcChatText = text;
	}


	public static string RenderMoney( int money ) {
		string render = "";
		string label_copper = Lang.inter[18];
		string label_silver = Lang.inter[17];
		string label_gold = Lang.inter[16];
		string label_plat = Lang.inter[15];

		int plat = 0;
		int gold = 0;
		int silver = 0;
		int copper = 0;

		if( money < 0 ) { money = 0; }

		if( money >= 1000000 ) {
			plat = money / 1000000;
			money -= plat * 1000000;
		}
		if( money >= 10000 ) {
			gold = money / 10000;
			money -= gold * 10000;
		}
		if( money >= 100 ) {
			silver = money / 100;
			money -= silver * 100;
		}
		if( money >= 1 ) {
			copper = money;
		}

		if( plat > 0 ) { render += plat + " " + label_plat; }
		if( render.Length > 0 ) { render += " "; }
		if( gold > 0 ) { render += gold + " " + label_gold; }
		if( render.Length > 0 ) { render += " "; }
		if( silver > 0 ) { render += silver + " " + label_silver; }
		if( render.Length > 0 ) { render += " "; }
		if( copper > 0 ) { render += copper + " " + label_copper; }

		return render;
	}


	public static void UpdateSoundForCurrentListener( SoundEffectInstance my_sound, int world_x, int world_y, float sound_range_scale=1f ) {
		try {
			int world_range_x = (int)(Main.screenPosition.X - (float)Main.screenWidth * 2f);
			int world_range_y = (int)(Main.screenPosition.Y - (float)Main.screenHeight * 2f);
			int range_width = Main.screenWidth * 5;
			int range_height = Main.screenHeight * 5;
			Rectangle screen_range = new Rectangle( world_range_x, world_range_y, range_width, range_height );
			Rectangle sound_source_point = new Rectangle( world_x, world_y, 1, 1 );

			// Sound is out of range?
			if( !sound_source_point.Intersects( screen_range ) ) {
				my_sound.Stop();
				return;
			} else if( my_sound.State == SoundState.Stopped ) {
				my_sound.Play();
			}

			float listener_x = Main.screenPosition.X + (float)Main.screenWidth / 2f;
			float listener_y = Main.screenPosition.Y + (float)Main.screenHeight / 2f;
			Vector2 listener_point = new Vector2( listener_x, listener_y );

			float x_dist = (float)world_x - listener_point.X;
			float y_dist = (float)world_y - listener_point.Y;
			float sound_range_limit = (float)Main.screenWidth * 1.5f * sound_range_scale;
			float abs_dist = (float)Math.Sqrt( (double)(x_dist * x_dist + y_dist * y_dist) );
			float volume = abs_dist / sound_range_limit;
			volume = Math.Min( 1f - volume, 1f );

			float x_offset_pan_unit = (float)Main.screenWidth / 2f;
			float x_offset = (float)world_x - listener_point.X;
			float x_offset_in_scrn_units = x_offset / x_offset_pan_unit;
			float pan = -MathHelper.Clamp( x_offset_in_scrn_units, -1f, 1f );

			volume *= Main.soundVolume;
			volume = MathHelper.Clamp( volume, 0f, 1f );

			if( volume > 0f ) {
				my_sound.Volume = volume;
				my_sound.Pan = pan;
			}

			if( volume > 0f ) {
				if( my_sound.State == SoundState.Stopped ) { my_sound.Play(); }
				my_sound.Volume = volume;
				my_sound.Pan = pan;
			} else {
				my_sound.Stop();
			}
		} catch( System.Exception ) { }
		return;
	}


	public static Vector2 PickRandomTileInPixelUnitRange( Rectangle pixel_unit_rect ) {
		Rectangle world_rect = new Rectangle( pixel_unit_rect.X / 16, pixel_unit_rect.Y / 16, pixel_unit_rect.Width / 16, pixel_unit_rect.Height / 16 );

		int width = (int)(world_rect.Right - world_rect.Left);
		int height = (int)(world_rect.Right - world_rect.Left);

		int tiles = width * height;

		int rand_tile = Main.rand.Next( tiles );

		int rand_y = rand_tile / width;
		int rand_x = rand_tile - (rand_y * width);

		return new Vector2( world_rect.X+(float)rand_x, world_rect.Y+(float)rand_y );
	}


	/**	Toggles a given light tile. Ported from terraria source. */
	public static bool TileLightToggle( int x, int y ) {
		//if( Main.tile.GetLength( 0 ) <= x || Main.tile.GetLength( 1 ) <= y ) { return; }	// Fail hard; parameters are always assumed correct
		Tile tile = Main.tile[x, y];

		int y_off;
		int x_off;
		short x_shift;
		//short y_shift;

		switch( tile.type ) {
		// Torch
		case 4:
			if( tile.frameX < 66 ) {
				tile.frameX += 66;
			} else {
				tile.frameX -= 66;
			}
			NetMessage.SendTileSquare( -1, x, y, 1 );
			break;

		// Christmas light
		case 149:
			if (tile.frameX < 54) {
				tile.frameX += 54;
			} else {
				tile.frameX -= 54;
			}
			NetMessage.SendTileSquare( -1, x, y, 1 );
			break;

		// Lantern
		case 42:
			for( y_off = (int)(tile.frameY / 18); y_off >= 2; y_off -= 2 ) { }
			y_off = y - y_off;

			x_shift = 18;
			if( tile.frameX > 0 ) {
				x_shift = -18;
			}
			Main.tile[x, y_off].frameX += x_shift;
			Main.tile[x, y_off + 1].frameX += x_shift;
			Wiring.SkipWire( x, y_off );
			Wiring.SkipWire( x, y_off + 1 );

			NetMessage.SendTileSquare( -1, x, y, 2 );
			break;

		// Tiki torch
		case 93:
			for( y_off = (int)(tile.frameY / 18); y_off >= 3; y_off -= 3 ) { }
			y_off = y - y_off;

			x_shift = 18;
			if( tile.frameX > 0 ) {
				x_shift = -18;
			}
			Main.tile[x, y_off].frameX += x_shift;
			Main.tile[x, y_off + 1].frameX += x_shift;
			Main.tile[x, y_off + 2].frameX += x_shift;
			Wiring.SkipWire( x, y_off );
			Wiring.SkipWire( x, y_off + 1 );
			Wiring.SkipWire( x, y_off + 2 );

			NetMessage.SendTileSquare( -1, x, y_off + 1, 3 );
			break;

		// Disco Ball
		case 126:
		// Chinese Lantern
		case 95:
		// Candelabra
		case 100:
		// Candelabra
		case 173:
			for( y_off = (int)(tile.frameY / 18); y_off >= 2; y_off -= 2 ) { }
			y_off = y - y_off;

			x_off = (int)(tile.frameX / 18);
			if( x_off > 1 ) {
				x_off -= 2;
			}
			x_off = x - x_off;

			x_shift = 36;
			if( Main.tile[x_off, y_off].frameX > 0 ) {
				x_shift = -36;
			}

			Main.tile[x_off, y_off].frameX += x_shift;
			Main.tile[x_off, y_off + 1].frameX += x_shift;
			Main.tile[x_off + 1, y_off].frameX += x_shift;
			Main.tile[x_off + 1, y_off + 1].frameX += x_shift;
			Wiring.SkipWire( x_off, y_off );
			Wiring.SkipWire( x_off + 1, y_off );
			Wiring.SkipWire( x_off, y_off + 1 );
			Wiring.SkipWire( x_off + 1, y_off + 1 );

			NetMessage.SendTileSquare( -1, x_off, y_off, 3 );
			break;

		// 	Chandelier
		case 34:
			for( y_off = (int)(tile.frameY / 18); y_off >= 3; y_off -= 3 ) { }
			y_off = y - y_off;

			x_off = (int)(tile.frameX / 18);
			if( x_off > 2 ) {
				x_off -= 3;
			}
			x_off = x - x_off;

			x_shift = 54;
			if( Main.tile[x_off, y_off].frameX > 0 ) {
				x_shift = -54;
			}

			for( int x_iter = x_off; x_iter < x_off + 3; x_iter++ ) {
				for( int y_iter = y_off; y_iter < y_off + 3; y_iter++ ) {
					Main.tile[x_iter, y_iter].frameX += x_shift;
					Wiring.SkipWire( x_iter, y_iter );
				}
			}

			NetMessage.SendTileSquare( -1, x_off + 1, y_off + 1, 3 );
			break;

		// Candle
		case 33:
		// Platinum Candle
		case 174:
			x_shift = 18;
			if( tile.frameX > 0 ) {
				x_shift = -18;
			}

			tile.frameX += x_shift;

			NetMessage.SendTileSquare( -1, x, y, 3 );
			break;

		// Lamp Post
		case 92:
			y_off = y - (int)(tile.frameY / 18);

			x_shift = 18;
			if( tile.frameX > 0 ) {
				x_shift = -18;
			}

			for( int y_iter = y_off; y_iter < y_off + 6; y_iter++ ) {
				Main.tile[x, y_iter].frameX += x_shift;
				Wiring.SkipWire( x, y_iter );
			}

			NetMessage.SendTileSquare( -1, x, y_off + 3, 7 );
			break;

		default:
			return false;
		}

		//WorldGen.TileFrame( x, y );
		return true;
	}



	/**	Turns a given light tile off. Ported from terraria source. */
	public static bool TileLightOff( int x, int y ) {
		//if( Main.tile.GetLength( 0 ) <= x || Main.tile.GetLength( 1 ) <= y ) { return; }	// Fail hard; parameters are always assumed correct
		Tile tile = Main.tile[x, y];

		int y_off;
		int x_off;
		short x_shift;
		//short y_shift;

		switch( tile.type ) {
		// Torch
		case 4:
			if( tile.frameX >= 66 ) { return false; }
			Tile tile1 = tile;
			Tile tile2 = tile1;
			tile2.frameX += 66;
			NetMessage.SendTileSquare( -1, x, y, 1 );
			break;
			
		// Christmas light
		case 149:
			if( tile.frameX >= 54 ) { return false; }
			tile.frameX += 54;
			NetMessage.SendTileSquare( -1, x, y, 1 );
			break;

		// Lantern
		case 42:
			if( tile.frameX > 0 ) { return false; }

			for( y_off = (int)(tile.frameY / 18); y_off >= 2; y_off -= 2 ) { }
			y_off = y - y_off;

			x_shift = 18;

			Main.tile[x, y_off].frameX += x_shift;
			Main.tile[x, y_off + 1].frameX += x_shift;
			Wiring.SkipWire( x, y_off );
			Wiring.SkipWire( x, y_off + 1 );

			NetMessage.SendTileSquare( -1, x, y, 2 );
			break;

		// Tiki torch
		case 93:
			if( tile.frameX > 0 ) { return false; }

			for( y_off = (int)(tile.frameY / 18); y_off >= 3; y_off -= 3 ) { }
			y_off = y - y_off;

			x_shift = 18;

			Main.tile[x, y_off].frameX += x_shift;
			Main.tile[x, y_off + 1].frameX += x_shift;
			Main.tile[x, y_off + 2].frameX += x_shift;
			Wiring.SkipWire( x, y_off );
			Wiring.SkipWire( x, y_off + 1 );
			Wiring.SkipWire( x, y_off + 2 );

			NetMessage.SendTileSquare( -1, x, y_off + 1, 3 );
			break;

		// Disco Ball
		case 126:
		// Chinese Lantern
		case 95:
		// Candelabra
		case 100:
		// Candelabra
		case 173:
			for( y_off = (int)(tile.frameY / 18); y_off >= 2; y_off -= 2 ) { }
			y_off = y - y_off;

			x_off = (int)(tile.frameX / 18);
			if( x_off > 1 ) { x_off -= 2; }
			x_off = x - x_off;

			if( Main.tile[x_off, y_off].frameX > 0 ) { return false; }

			x_shift = 36;

			Main.tile[x_off, y_off].frameX += x_shift;
			Main.tile[x_off, y_off + 1].frameX += x_shift;
			Main.tile[x_off + 1, y_off].frameX += x_shift;
			Main.tile[x_off + 1, y_off + 1].frameX += x_shift;
			Wiring.SkipWire( x_off, y_off );
			Wiring.SkipWire( x_off + 1, y_off );
			Wiring.SkipWire( x_off, y_off + 1 );
			Wiring.SkipWire( x_off + 1, y_off + 1 );

			NetMessage.SendTileSquare( -1, x_off, y_off, 3 );
			break;

		// 	Chandelier
		case 34:
			for( y_off = (int)(tile.frameY / 18); y_off >= 3; y_off -= 3 ) { }
			y_off = y - y_off;

			x_off = (int)(tile.frameX / 18);
			if( x_off > 2 ) { x_off -= 3; }
			x_off = x - x_off;

			if( Main.tile[x_off, y_off].frameX > 0 ) { return false; }

			x_shift = 54;

			for( int x_iter = x_off; x_iter < x_off + 3; x_iter++ ) {
				for( int y_iter = y_off; y_iter < y_off + 3; y_iter++ ) {
					Main.tile[x_iter, y_iter].frameX += x_shift;
					Wiring.SkipWire( x_iter, y_iter );
				}
			}

			NetMessage.SendTileSquare( -1, x_off + 1, y_off + 1, 3 );
			break;

		// Candle
		case 33:
		// Platinum Candle
		case 174:
			if( tile.frameX > 0 ) { return false; }

			x_shift = 18;

			tile.frameX += x_shift;

			NetMessage.SendTileSquare( -1, x, y, 3 );
			break;

		// Lamp Post
		case 92:
			if( tile.frameX > 0 ) { return false; }

			y_off = y - (int)(tile.frameY / 18);

			x_shift = 18;

			for( int y_iter = y_off; y_iter < y_off + 6; y_iter++ ) {
				Main.tile[x, y_iter].frameX += x_shift;
				Wiring.SkipWire( x, y_iter );
			}

			NetMessage.SendTileSquare( -1, x, y_off + 3, 7 );
			break;
			
		default:
			return false;
		}

		return true;
	}


	public static int GetTileStyle2xX( Tile tile ) {
		return (int)tile.frameX / 36;
	}

	public static int GetTileStyleTorch( Tile tile ) {
		return (int)tile.frameY / 22;
	}


	public static Tile GetNextScreenTileOfType( int type, ref int x_where, ref int y_where ) {
		if( WorldGen.tileCount[type] == 0 ) { return null; }

		int min_x = (int)(Main.screenPosition.X / 16f - 1f);
		int min_y = (int)(Main.screenPosition.Y / 16f - 1f);
		int max_x = (int)((Main.screenPosition.X + (float)Main.screenWidth) / 16f) + 2;
		int max_y = (int)((Main.screenPosition.Y + (float)Main.screenHeight) / 16f) + 2;

		if( x_where < min_x ) { x_where = min_x; }
		if( y_where < min_y ) { y_where = min_y-1; }	// Ew

		for( ; x_where < max_x; x_where++ ) {
			for( y_where++; y_where < max_y; y_where++ ) {	// Ew
				Tile tile = Main.tile[ x_where, y_where ];
				if( tile == null || !tile.active() ) { continue; }

				if( tile.type == type ) {
					return tile;
				}
			}

			y_where = min_y - 1;	// Ew
		}

		x_where = -1;
		y_where = -1;
		return null;
	}

	//public static List<Tile> GetScreenTilesOfType( int type ) {
	//    List<Tile> list = new List<Tile>();

	//    if( WorldGen.tileCount[type] == 0 ) { return list; }

	//    int min_x = (int)(Main.screenPosition.X / 16f - 1f);
	//    int max_x = (int)((Main.screenPosition.X + (float)Main.screenWidth) / 16f) + 2;
	//    int min_y = (int)(Main.screenPosition.Y / 16f - 1f);
	//    int max_y = (int)((Main.screenPosition.Y + (float)Main.screenHeight) / 16f) + 2;

	//    for( int i = min_x; i < max_x; i++ ) {
	//        for( int j = min_y; j < max_y; j++ ) {
	//            Tile tile = Main.tile[i, j];
	//            if( tile == null ) { continue; }
	//            if( !tile.active() ) { continue; }
	//            if( tile.type == type ) { list.Add( tile ); }
	//        }
	//    }

	//    return list;
	//}



	public static bool IsKillerByName( string death_text, string killer_name ) {
		string[] death_parts = death_text.Split( ' ' );
		string[] killer_parts = killer_name.Split( ' ' );

		killer_parts[killer_parts.Length - 1] += ".";

		bool is_killer = true;

		for( int i = 1; i <= death_parts.Length; i++ ) {
			if( killer_parts.Length < i ) {
				is_killer = true;
				break;
			}

			string death_word = death_parts[death_parts.Length - i];
			string killer_word = killer_parts[killer_parts.Length - i];

			if( death_word != killer_word ) {
				is_killer = false;
				break;
			}
		}

		return is_killer;
	}


	/**	Gets a map of buff types to buff indexes. */
	public static Dictionary<int, int> GetBuffs( Player player ) {
		Dictionary<int, int> map = new Dictionary<int, int>();
		for( int i = 0; i < player.maxBuffs; i++ ) {
			if( player.buffType[i] != 0 && player.buffTime[i] > 0 ) {
				map[ player.buffType[i] ] = i;
			}
		}
		return map;
	}

	/**	Gets a map of buff types to buff indexes. */
	public static Dictionary<int, int> GetBuffs( NPC npc ) {
		Dictionary<int, int> map = new Dictionary<int, int>();
		for( int i = 0; i < npc.maxBuffs; i++ ) {
			if( npc.buffType[i] != 0 ) {
				map[ npc.buffType[i] ] = i;
			}
		}
		return map;
	}


	public static Item[] GetArmors( Player player ) {
		Item[] armors = new Item[3] { player.armor[0], player.armor[1], player.armor[2] };
		return armors;
	}

	public static Item[] GetAccessories( Player player ) {
		Item[] accessories = new Item[5] { player.armor[3], player.armor[4], player.armor[5], player.armor[6], player.armor[7] };
		return accessories;
	}

	public static Item[] GetVanities( Player player ) {
		Item[] vanities = new Item[3] { player.armor[8], player.armor[9], player.armor[10] };
		return vanities;
	}

	public static Item[] GetVanityAccessories( Player player ) {
		Item[] vanity_acc = new Item[5] { player.armor[11], player.armor[12], player.armor[13], player.armor[14], player.armor[15] };
		return vanity_acc;
	}


	public static bool IsPlayerNaked( Player player ) {
		// Armor
		if( !player.armor[0].IsBlank() ) { return false; }
		if( !player.armor[1].IsBlank() ) { return false; }
		if( !player.armor[2].IsBlank() ) { return false; }
		// Accessory
		if( !player.armor[3].IsBlank() && !player.hideVisual[3] ) { return false; }
		if( !player.armor[4].IsBlank() && !player.hideVisual[4] ) { return false; }
		if( !player.armor[5].IsBlank() && !player.hideVisual[5] ) { return false; }
		if( !player.armor[6].IsBlank() && !player.hideVisual[6] ) { return false; }
		if( !player.armor[7].IsBlank() && !player.hideVisual[7] ) { return false; }
		// Vanity
		if( !player.armor[8].IsBlank() ) { return false; }
		if( !player.armor[9].IsBlank() ) { return false; }
		if( !player.armor[10].IsBlank() ) { return false; }
		// Vanity Accessory
		if( !player.armor[11].IsBlank() /*&& !player.hideVisual[3]*/ ) { return false; }
		if( !player.armor[12].IsBlank() /*&& !player.hideVisual[4]*/ ) { return false; }
		if( !player.armor[13].IsBlank() /*&& !player.hideVisual[5]*/ ) { return false; }
		if( !player.armor[14].IsBlank() /*&& !player.hideVisual[6]*/ ) { return false; }
		if( !player.armor[15].IsBlank() /*&& !player.hideVisual[7]*/ ) { return false; }

		return true;
	}


	public static bool IsPlayerInvisible( Player player ) {
		//if( player.dripping || player.drippingSlime || player.onFire || drawPlayer.onFire2 ) { return false; }
		//if( player.inferno ) { return false; }	// TODO: Make this list comprehensive. Until then, won't bother.
		if( !GetBuffs( player ).ContainsKey( BuffDef.byName["Vanilla:Invisibility"] ) ) { return false; }
		if( !IsPlayerNaked( player ) ) { return false; }

		return true;
	}
}
